@@ -20,6 +20,11 @@
 # 02110-1301, USA.
 #
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import print_function
+from __future__ import unicode_literals
+
 import errno
 import optparse
 import os
@@ -250,7 +255,7 @@ def passthrough_gir(path, f):
     parser.parse(path)
 
     writer = GIRWriter(parser.get_namespace())
-    f.write(writer.get_xml())
+    f.write(writer.get_encoded_xml())
 
 
 def test_codegen(optstring,
@@ -271,7 +276,7 @@ def test_codegen(optstring,
                                       include_last_src)
         gen.write()
     else:
-        _error("Invaild namespace %r" % (namespace, ))
+        _error("Invaild namespace '%s'" % (namespace, ))
     return 0
 
 
@@ -293,6 +298,7 @@ def process_packages(options, packages):
         # the error output should have already appeared on our stderr,
         # so we just exit
         return 1
+    output = output.decode('ascii')
     # Some pkg-config files on Windows have options we don't understand,
     # so we explicitly filter to only the ones we need.
     options_whitelist = ['-I', '-D', '-U', '-l', '-L']
@@ -342,8 +348,8 @@ def extract_filelist(options):
 
 def create_namespace(options):
     if options.strip_prefix:
-        print """g-ir-scanner: warning: Option --strip-prefix has been deprecated;
-see --identifier-prefix and --symbol-prefix."""
+        print("""g-ir-scanner: warning: Option --strip-prefix has been deprecated;
+see --identifier-prefix and --symbol-prefix.""")
         options.identifier_prefixes.append(options.strip_prefix)
 
     # We do this dance because the empty list has different semantics from
@@ -381,11 +387,11 @@ def create_transformer(namespace, options):
 
     for include in options.includes:
         if os.sep in include:
-            _error("Invalid include path %r" % (include, ))
+            _error("Invalid include path '%s'" % (include, ))
         try:
             include_obj = Include.from_string(include)
         except:
-            _error("Malformed include %r\n" % (include, ))
+            _error("Malformed include '%s'\n" % (include, ))
         transformer.register_include(include_obj)
     for include_path in options.includes_uninstalled:
         transformer.register_include_uninstalled(include_path)
@@ -439,18 +445,19 @@ def create_source_scanner(options, args):
 
 
 def write_output(data, options):
+    """Write encoded XML 'data' to the filename specified in 'options'."""
     if options.output == "-":
         output = sys.stdout
     elif options.reparse_validate_gir:
         main_f, main_f_name = tempfile.mkstemp(suffix='.gir')
-        with os.fdopen(main_f, 'w') as main_f:
+        with os.fdopen(main_f, 'wb') as main_f:
             main_f.write(data)
 
         temp_f, temp_f_name = tempfile.mkstemp(suffix='.gir')
-        with os.fdopen(temp_f, 'w') as temp_f:
+        with os.fdopen(temp_f, 'wb') as temp_f:
             passthrough_gir(main_f_name, temp_f)
         if not utils.files_are_identical(main_f_name, temp_f_name):
-            _error("Failed to re-parse gir file; scanned=%r passthrough=%r" % (
+            _error("Failed to re-parse gir file; scanned='%s' passthrough='%s'" % (
                 main_f_name, temp_f_name))
         os.unlink(temp_f_name)
         try:
@@ -463,7 +470,7 @@ def write_output(data, options):
         return 0
     else:
         try:
-            output = open(options.output, "w")
+            output = open(options.output, 'wb')
         except IOError as e:
             _error("opening output for writing: %s" % (e.strerror, ))
 
@@ -546,8 +553,9 @@ def scanner_main(args):
         message.fatal("warnings configured as fatal")
         return 1
     elif warning_count > 0 and options.warn_all is False:
-        print ("g-ir-scanner: %s: warning: %d warnings suppressed (use --warn-all to see them)"
-               % (transformer.namespace.name, warning_count, ))
+        print("g-ir-scanner: %s: warning: %d warnings suppressed "
+              "(use --warn-all to see them)" %
+              (transformer.namespace.name, warning_count, ))
 
     # Write out AST
     if options.packages_export:
@@ -558,7 +566,7 @@ def scanner_main(args):
     transformer.namespace.c_includes = options.c_includes
     transformer.namespace.exported_packages = exported_packages
     writer = Writer(transformer.namespace)
-    data = writer.get_xml()
+    data = writer.get_encoded_xml()
 
     write_output(data, options)
 
