@@ -25,6 +25,30 @@ regress_set_abort_on_error (gboolean in)
   abort_on_error = in;
 }
 
+/* return annotations */
+
+/**
+ * regress_test_return_allow_none:
+ *
+ * Returns: (allow-none):
+ */
+char *
+regress_test_return_allow_none (void)
+{
+  return NULL;
+}
+
+/**
+ * regress_test_return_nullable:
+ *
+ * Returns: (nullable):
+ */
+char *
+regress_test_return_nullable (void)
+{
+  return NULL;
+}
+
 /* basic types */
 gboolean
 regress_test_boolean (gboolean in)
@@ -496,6 +520,7 @@ regress_test_utf8_inout (char **inout)
 {
   /* inout parameter, transfer mode full */
   g_assert (strcmp (*inout, utf8_const) == 0);
+  g_free (*inout);
   *inout = g_strdup (utf8_nonconst);
 }
 
@@ -639,6 +664,8 @@ regress_test_array_int_inout (int *n_ints, int **ints)
       new_ints = g_malloc(sizeof(**ints) * *n_ints);
       for (i = 0; i < *n_ints; i++)
 	new_ints[i] = (*ints)[i + 1] + 1;
+
+      g_free (*ints);
       *ints = new_ints;
     }
 }
@@ -1272,34 +1299,38 @@ static const gchar *string_array[] = {
 GHashTable *
 regress_test_ghash_gvalue_return (void)
 {
-  GHashTable *hash;
-  GValue *value;
-  hash = g_hash_table_new_full(g_str_hash, g_str_equal,
-                               g_free, (GDestroyNotify)g_value_free);
+  static GHashTable *hash = NULL;
 
-  value = g_value_new(G_TYPE_INT);
-  g_value_set_int(value, 12);
-  g_hash_table_insert(hash, g_strdup("integer"), value);
+  if (hash == NULL)
+    {
+      GValue *value;
+      hash = g_hash_table_new_full(g_str_hash, g_str_equal,
+                                   g_free, (GDestroyNotify)g_value_free);
 
-  value = g_value_new(G_TYPE_BOOLEAN);
-  g_value_set_boolean(value, TRUE);
-  g_hash_table_insert(hash, g_strdup("boolean"), value);
+      value = g_value_new(G_TYPE_INT);
+      g_value_set_int(value, 12);
+      g_hash_table_insert(hash, g_strdup("integer"), value);
 
-  value = g_value_new(G_TYPE_STRING);
-  g_value_set_string(value, "some text");
-  g_hash_table_insert(hash, g_strdup("string"), value);
+      value = g_value_new(G_TYPE_BOOLEAN);
+      g_value_set_boolean(value, TRUE);
+      g_hash_table_insert(hash, g_strdup("boolean"), value);
 
-  value = g_value_new(G_TYPE_STRV);
-  g_value_set_boxed(value, string_array);
-  g_hash_table_insert(hash, g_strdup("strings"), value);
+      value = g_value_new(G_TYPE_STRING);
+      g_value_set_string(value, "some text");
+      g_hash_table_insert(hash, g_strdup("string"), value);
 
-  value = g_value_new(REGRESS_TEST_TYPE_FLAGS);
-  g_value_set_flags(value, REGRESS_TEST_FLAG1 | REGRESS_TEST_FLAG3);
-  g_hash_table_insert(hash, g_strdup("flags"), value);
+      value = g_value_new(G_TYPE_STRV);
+      g_value_set_boxed(value, string_array);
+      g_hash_table_insert(hash, g_strdup("strings"), value);
 
-  value = g_value_new(regress_test_enum_get_type());
-  g_value_set_enum(value, REGRESS_TEST_VALUE2);
-  g_hash_table_insert(hash, g_strdup("enum"), value);
+      value = g_value_new(REGRESS_TEST_TYPE_FLAGS);
+      g_value_set_flags(value, REGRESS_TEST_FLAG1 | REGRESS_TEST_FLAG3);
+      g_hash_table_insert(hash, g_strdup("flags"), value);
+
+      value = g_value_new(regress_test_enum_get_type());
+      g_value_set_enum(value, REGRESS_TEST_VALUE2);
+      g_hash_table_insert(hash, g_strdup("enum"), value);
+    }
 
   return hash;
 }
@@ -3451,9 +3482,24 @@ regress_test_simple_callback (RegressTestSimpleCallback callback)
     return;
 }
 
+/**
+ * regress_test_noptr_callback:
+ * @callback: (scope call) (allow-none):
+ *
+ **/
+void
+regress_test_noptr_callback (RegressTestNoPtrCallback callback)
+{
+    if (callback != NULL)
+        callback();
+
+    return;
+}
+
 /**
  * regress_test_callback_user_data:
  * @callback: (scope call):
+ * @user_data: (not nullable):
  *
  * Call - callback parameter persists for the duration of the method
  * call and can be released on return.
@@ -3600,6 +3646,7 @@ regress_test_async_ready_callback (GAsyncReadyCallback callback)
   GSimpleAsyncResult *result = g_simple_async_result_new (NULL, callback, NULL,
     regress_test_async_ready_callback);
   g_simple_async_result_complete_in_idle (result);
+  g_object_unref (result);
   G_GNUC_END_IGNORE_DEPRECATIONS
 }
 
@@ -4112,3 +4159,14 @@ regress_like_xkl_config_item_set_name (RegressLikeXklConfigItem *self,
   strncpy (self->name, name, sizeof (self->name) - 1);
   self->name[sizeof(self->name)-1] = '\0';
 }
+
+/**
+ * regress_get_variant:
+ *
+ * Returns: (transfer floating): A new variant
+ */
+GVariant *
+regress_get_variant (void)
+{
+  return g_variant_new_int32 (42);
+}
