@@ -15,6 +15,13 @@
 _GI_TEST_EXTERN
 void regress_set_abort_on_error (gboolean abort_on_error);
 
+/* return annotations */
+_GI_TEST_EXTERN
+char *regress_test_return_allow_none (void);
+
+_GI_TEST_EXTERN
+char *regress_test_return_nullable (void);
+
 /* basic types */
 _GI_TEST_EXTERN
 gboolean regress_test_boolean (gboolean in);
@@ -918,6 +925,8 @@ void regress_func_obj_null_in (RegressTestObj *obj);
 _GI_TEST_EXTERN
 void regress_test_obj_null_out (RegressTestObj **obj);
 
+void regress_func_obj_nullable_in (RegressTestObj *obj);
+
 /* inheritance */
 #define REGRESS_TEST_TYPE_SUB_OBJ           (regress_test_sub_obj_get_type ())
 #define REGRESS_TEST_SUB_OBJECT(object)     (G_TYPE_CHECK_INSTANCE_CAST ((object), REGRESS_TEST_TYPE_SUB_OBJ, RegressTestSubObj))
@@ -1024,6 +1033,7 @@ regress_test_fundamental_sub_object_new (const char *data);
 
 /* callback */
 typedef void (*RegressTestSimpleCallback) (void);
+typedef void RegressTestNoPtrCallback (void);
 typedef int (*RegressTestCallback) (void);
 typedef int (*RegressTestCallbackUserData) (gpointer user_data);
 /**
@@ -1067,6 +1077,9 @@ typedef void (* RegressTestCallbackArrayInOut) (int **ints, int *length);
 _GI_TEST_EXTERN
 void regress_test_simple_callback (RegressTestSimpleCallback callback);
 
+_GI_TEST_EXTERN
+void regress_test_noptr_callback (RegressTestNoPtrCallback callback);
+
 _GI_TEST_EXTERN
 int regress_test_callback (RegressTestCallback callback);
 
@@ -1423,4 +1436,7 @@ typedef struct {
   };
 } RegressAnAnonymousUnion;
 
+GVariant *
+regress_get_variant (void);
+
 #endif /* __GITESTTYPES_H__ */
